if (!window.launchpad) {
    window.launchpad = {};
}
if (!window.launchpad.connectExisting) {
    window.launchpad.connectExisting = {};
}

launchpad.connectExisting = {

    createChildPhysicalIdsPromise: async function (childId) {
        try {
            const options = {
                url: App.ajax.getUrl('macroResolver', 'getResolvedValue'),
                data: JSON.stringify([['physicalid'], childId])
            };
            return Promise.resolve(App.ajax.call(options, false));
        } catch (err) {
            return Promise.reject(err);
        }
    },

    createOperationsChildrenPromise: async function (parentId) {
        try {
            const options = {
                url: App.ajax.getUrl('macroResolver', 'getResolvedValue'),
                data: JSON.stringify([['physicalid', 'from[VPMInstance].physicalid', 'from[VPMRepInstance].physicalid'], parentId])
            };
            return Promise.resolve(App.ajax.call(options, false));
        } catch (err) {
            return Promise.reject(err);
        }
    },

    createConnectPayloadFull: function (input, childReslovedPhysicalId, operationsChildrenPayload) {
        try {
            // prepare data array
            let indexOflist = 0;
            const parentsPhysicalIds = input.contextOids.map((parentId) => {
                return {
                    cacheId: indexOflist++,
                    pid: input.datatable.objects[parentId]?.data?.name?.values[0]?.objectId ?? parentId
                };
            });

            indexOflist = 0;
            const childsPhysicalIds = [];
            input.selectedOids.forEach(child => {
                let tempChildid = input.datatable.objects[child]?.data?.name?.values[0]?.objectId ?? child;
                if (tempChildid.indexOf('.') === -1) {
                    childsPhysicalIds.push({ cacheId: indexOflist++, pid: tempChildid });
                }
            });

            //add resolved child physicalids
            childReslovedPhysicalId?.forEach(childPhysicalId => {
                childsPhysicalIds.push({ cacheId: indexOflist++, pid: childPhysicalId.at(0) });
            });

            // create data structure -> or it will be physicalId or it will be cacheId
            indexOflist = 0;
            const operationsRows = [];
            let isParentFirstRound = true;
            let isChildFirstRound = true;
            parentsPhysicalIds.forEach(parentObject => {
                isParentFirstRound = true;
                const operationChildren = [];
                operationsChildrenPayload.forEach(connectionsData => {
                    if (connectionsData.pid === parentObject.pid) {
                        connectionsData.children.forEach(connectedChild => {
                            operationChildren.push(connectedChild);
                        });
                    }
                });
                childsPhysicalIds.forEach(childObject => {
                    parentSection = {};
                    childSection = {};
                    if (isParentFirstRound) {
                        parentSection.isInstanceOf = parentObject.pid;
                        if (operationChildren.length > 0) {
                            parentSection.children = operationChildren.slice();
                        }
                    }

                    if (isChildFirstRound) {
                        childSection.isInstanceOf = childObject.pid;
                    }

                    parentSection.cacheId = parentObject.cacheId;
                    childSection.cacheId = childObject.cacheId;

                    operationsRows.push({ parent: parentSection, child: childSection });
                    isParentFirstRound = false;
                });
                isChildFirstRound = false;
            });

            const newPayload = {
                "version": "1.0",
                "bAllOrNothing": false,
                "operations": operationsRows
            };
            return newPayload;
        } catch (error) {
            App.log.error({ title: `createConnectPayloadFull`, message: error }, {}, error);

        }
    },

    //get childs without physicalid
    createConnectPayload: async function (input, operationsChildrenPayload) {
        try {
            return new Promise((resolve, reject) => {
                const childsIds = [];
                input.selectedOids.forEach(child => {
                    let tempChildid = input.datatable.objects[child]?.data?.name?.values[0]?.objectId ?? child;
                    if (tempChildid.indexOf('.') !== -1) {
                        childsIds.push(tempChildid);
                    }
                });

                let childReslovedPhysicalId;

                if (childsIds.length > 0) {
                    const childPhysicalIdsPromises = [];
                    childsIds.forEach(childid => {
                        childPhysicalIdsPromises.push(launchpad.connectExisting.createChildPhysicalIdsPromise(childid));
                    })

                    Promise.all(childPhysicalIdsPromises)
                        .then(response => {
                            childReslovedPhysicalId = response.map(childPhysicalId => {
                                return childPhysicalId;
                            });
                            resolve(launchpad.connectExisting.createConnectPayloadFull(input, childReslovedPhysicalId, operationsChildrenPayload));
                        })
                        .catch(errorMessage => {
                            const errMessage = launchpad.lpadUtils.convertStringToObject(errorMessage);
                            reject(errMessage);
                            throw (errMessage);
                        });
                } else {
                    resolve(launchpad.connectExisting.createConnectPayloadFull(input, childReslovedPhysicalId, operationsChildrenPayload));
                }
            });
        } catch (error) {
            App.log.error({ title: `createConnectPayload`, message: error }, {}, error);
        }
    },

    // get relationship data for parents for element operations/children in Payload
    createOperationsChildren: async function (input) {
        try {
            return new Promise((resolve, reject) => {
                const parentOperationsChildren = [];
                const parentIdsForConectionsPromises = [];

                input.contextOids.forEach((parentId) => {
                    parentIdsForConectionsPromises.push(launchpad.connectExisting.createOperationsChildrenPromise(parentId));
                });

                Promise.all(parentIdsForConectionsPromises)
                    .then(response => {
                        response.forEach(data => {
                            let connectionsData = [];
                            if (data.at(1) !== null) {
                                data.at(1).split('\x07').forEach(element => connectionsData.push(element));
                            }
                            if (data.at(2) !== null) {
                                data.at(2).split('\x07').forEach(element => connectionsData.push(element));
                            }
                            if (connectionsData.length > 0) {
                                parentOperationsChildren.push({ pid: data.at(0), children: connectionsData });
                            }
                        });
                        resolve(parentOperationsChildren);
                    })
                    .catch(errorMessage => {
                        const errMessage = launchpad.lpadUtils.convertStringToObject(errorMessage);
                        reject(errMessage);
                        throw (errMessage);
                    });
            });
        } catch (error) {
            App.log.error({ title: `createOperationsChildren`, message: error }, {}, error);
        }
    },

    callServiceForConnectExisting: async function (securityContext, payloadConnectObjects) {
        try {
            return new Promise((resolve, reject) => {
                const optionsConnectObjects = {
                    url: `${App.info.contextURL}resources/product/instances/?securityContext=ctx::${securityContext}&tenant=OnPremise`,
                    contentType: App.ajax.contentType.JSON,
                    headers: { SecurityContext: `ctx::${securityContext}`, ContentType: App.ajax.contentType.JSON },
                    method: App.ajax.requestType.POST,
                    data: JSON.stringify(payloadConnectObjects)
                };

                App.ajax.call(optionsConnectObjects)
                    .then(response => {
                        resolve(response);
                    })
                    .catch(errorMessage => {
                        const errMessage = launchpad.lpadUtils.convertStringToObject(errorMessage);
                        reject(errMessage);
                        throw (errMessage);
                    });
            });
        } catch (error) {
            App.log.error({ title: `callServiceForConnectExisting`, message: error }, {}, error);
        }
    },

    invokeWSForConnectExisting: async function (input) {
        try {
            this.loader.start();
            let thisSecurityContext = await launchpad.lpadUtils.getSecurityContext();
            const operationsChildrenPayload = await launchpad.connectExisting.createOperationsChildren(input);
            const payloadConnectObjects = await launchpad.connectExisting.createConnectPayload(input, operationsChildrenPayload);
            const resultMessageData = await launchpad.connectExisting.callServiceForConnectExisting(thisSecurityContext, payloadConnectObjects);

            this.loader.stop();
            this.destroy();

            App.log.debug('Connected RelationshipIds', {}, resultMessageData);

            let successMessage = "";
            let errorMessageFull = "";

            if (resultMessageData.results.length > 0) {
                resultMessageData.results.forEach(result => {
                    if (result.status === 'success') {
                        if (!successMessage.includes(result.instanceName)) {
                            successMessage += `<br><br> ${result.instanceName}`;
                        }
                    } else {
                        let errorMessage = "";
                        result.messages.forEach(message => {
                            if (!errorMessageFull.includes(message)) {
                                errorMessage += `${message} <br>`;
                            }
                        });
                        if (errorMessage) {
                            errorMessageFull += `<br><br> ${errorMessage}`;
                        }
                    }
                })
            }
            this.datatable.widget.refresh();

            if (successMessage) {
                App.toaster.success({ title: App.i18n.t("searchv2.dataTable.messages.objectsAddedSuccessfully"), message: successMessage });
            }
            if (errorMessageFull) {
                App.log.error({ title: App.i18n.t("searchv2.dataTable.messages.objectsConnectionError"), message: errorMessageFull }, {}, errorMessageFull);
            }

        } catch (e) {
            this.loader.stop();
            if (e) {
                App.log.error({ title: App.i18n.t("searchv2.dataTable.messages.objectsConnectionError"), message: launchpad.lpadUtils.getErrorMessage(e) }, {}, e);
            } else {
                App.log.error(App.i18n.t("searchv2.dataTable.messages.objectsConnectionError"), {}, "");
            }

        };

    },
};
