if (!window.launchpad) {
    window.launchpad = {};
}
if (!window.launchpad.lpadUtils) {
    window.launchpad.lpadUtils = {};
}
launchpad.lpadUtils = {
    createArrayFromList: function (inputList) {
        try {
            const returnArray = [];
            for (let index = 0; index < inputList.length; index++) {
                returnArray.push([inputList[index]]);
            }
            return returnArray;
        } catch (error) {
            App.log.error({ title: `launchpad.lpadUtils.createArrayFromList`, message: error }, {}, error);
        }
    },

    getCSRFToken: async function () {
        try {
            return new Promise((resolve, reject) => {
                const url = `${App.info.contextURL}resources/v1/application/CSRF`;
                const method = App.ajax.requestType.GET;
                App.ajax.call({ url, method })
                    .then(response => {
                        if (response.success) {
                            resolve(response);
                        } else {
                            reject(response);
                        }

                    })
                    .catch(errorMessage => {
                        reject(errorMessage);
                        if (!errorMessage) {
                            let errMessage = App.i18n.t("launchpad.message.errorGetCSRFToken");
                            App.log.error(errMessage, {}, errMessage);
                        }
                        throw (errorMessage);
                    });
            });
        } catch (error) {
            App.log.error({ title: `launchpad.lpadUtils.getCSRFToken`, message: error }, {}, error);
        }
    },

    convertStringToObject: function (inputObject) {
        try {
            if (typeof (inputObject) === 'string' && inputObject.length > 0) {
                return JSON.parse(inputObject);
            }
            return inputObject;
        } catch (error) {
            App.log.error({ title: `launchpad.lpadUtils.convertStringToObject`, message: error }, {}, error);
        }
    },

    getErrorMessage: function (inputObject) {
        try {
            let err = launchpad.lpadUtils.convertStringToObject(inputObject);
            const errKeys = Object.keys(err);
            if (errKeys.includes('error')) {
                return err.error;
            }
            if (errKeys.includes('messages')) {
                if (typeof (err.messages) === 'string') {
                    return err.messages;
                } else {
                    return err.messages[0];
                }
            }
            if (errKeys.includes('report')) {
                const reportErrKeys = Object.keys(err.report);
                if (reportErrKeys.includes('error')) {
                    return err.report[0].error;
                }
            }
            if (errKeys.includes('status')) {
                return err.status;
            }
            if (errKeys.includes('message')) {
                return err.message;
            }
            if (errKeys.includes('responseText')) {
                return err.responseText;
            }
            if (errKeys.includes('responseJSON')) {
                const responseJSONErrKeys = Object.keys(err.responseJSON);
                if (responseJSONErrKeys.includes('error')) {
                    return err.responseJSON.error;
                }
                if (responseJSONErrKeys.includes('errorMessage')) {
                    return err.responseJSON.errorMessage;
                }
            }
            return ('error');
        } catch (error) {
            App.log.error({ title: `launchpad.lpadUtils.getErrorMessage`, message: error }, {}, error);
        }
    },

    getSecurityContext: async function () {
        try {
            return new Promise((resolve, reject) => {

                const scRequestURL = App.info.contextURL + 'resources/pno/person/getsecuritycontext';
                const optionsGetSecurityContext = {
                    method: App.ajax.requestType.GET,
                    url: scRequestURL
                };
                App.ajax
                    .call(optionsGetSecurityContext, true, {}, false)
                    .then(response => {
                        let result = response;
                        if (typeof (response) !== 'object') {
                            result = JSON.parse(response);
                        }
                        resolve(result.SecurityContext);
                    }).catch(errorMessage => {
                        const errMessage = launchpad.lpadUtils.convertStringToObject(errorMessage);
                        reject(errMessage);
                        throw (errMessage);
                    });
            });
        } catch (error) {
            App.log.error({ title: `launchpad.lpadUtils.getSecurityContext`, message: error }, {}, error);
        }
    },

    getInfoForObjectId: async function (objectId, kindOfData, messageGetDataFailure) {
        try {
            return new Promise((resolve, reject) => {
                const options = {
                    url: App.ajax.getUrl('macroResolver', 'getResolvedValue'),
                    data: JSON.stringify([kindOfData, objectId])
                };

                App.ajax.call(options)
                    .then(response => {
                        resolve(response);
                    })
                    .catch(errorMessage => {
                        reject(errorMessage);
                        if (!errorMessage) {
                            let errMessage = App.i18n.t(`${messageGetDataFailure}`);
                            App.log.error(errMessage, {}, errMessage);
                        }
                        throw (errorMessage);
                    });
            });
        } catch (error) {
            App.log.error({ title: `launchpad.lpadUtils.getInfoForObjectId`, message: error }, {}, error);
        }
    },
};

